package glyph;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.awt.*;
import java.awt.event.*;
/**
 * This class represents the bidirectional interface between a view and
 * a model object being represented in the view.
 *
 * implements action listener to facilitate menu creation
 * Creation date: (08/02/2002 16:30:30)
 * @author: 
 */
public abstract class Node implements ActionListener {
	private static final String HIDE_GLYPH = "Hide Glyph";
	private static final String SHOW_GLYPH = "Show Glyph";
	private static final String SHOW_CHILDREN = "Show Children";
	private static final String MI_HELP = "Help";

	private static final String SHOW_BOUNDARY = "Show Boundary";
	private static final String HIDE_BOUNDARY = "Hide Boundary";
	
	/**
	 * by hard coding a matching glyph we ensure that there
	 * is always a valid value.
	 */
	private final AreaGlyph glyph = new AreaGlyph(this);
	private boolean visible = true;
/**
 * package private access to glyph
 */
final AreaGlyph _getGlyph() {
	return glyph;
}
public void actionPerformed(ActionEvent event) {
    String cmd = event.getActionCommand();
    if (HIDE_GLYPH.equals(cmd)) {
        visible = false;
        declareChanged();
    } else if (SHOW_GLYPH.equals(cmd)) {
        visible = true;
        declareChanged();
    } else if (SHOW_CHILDREN.equals(cmd)) {
        setChildrenVisible();
    } else if (MI_HELP.equals(cmd)) {
	    getView().showHelp(this, true);
    } else if (SHOW_BOUNDARY.equals(cmd)) {
	    glyph.setShowBoundary(true);
        declareChanged();
    } else if (HIDE_BOUNDARY.equals(cmd)) {
	    glyph.setShowBoundary(false);
        declareChanged();
    }
}
/**
 * add a child node if not already added
 */
public void addNode(Node node) {
	if (null == node.glyph.getParent()) {
	    // do not already have a parent
		AreaGlyph newGlyph = node._getGlyph();
		glyph.addChild(newGlyph);
	}
}
/**
 * called when receiver has been attached, and before redraw
 * this is for benefit of subclass notification
 */
protected void attached() {
}
/**
 * return true if the receivers representation can be
 * detached from its parent by dragging.  Subclasses should
 * override to prevent detach.
 */
protected boolean canDetach() {
	return true;
}
/**
 * declare that the receiver has changed in a way that requires
 * a redraw
 * This could be a header or body change.  This call is not required
 * for an addNode() or a delete().
 */
protected void declareChanged() {
	if (null != glyph) {
		// if we have not added the node to the view it will
		// not have a glyph....
		glyph.__changed();
	}
	if (null != getGlyphView()) {
		getGlyphView().repaint();
	}
}
/**
 * delete this node if not already deleted
 */
public void delete() {
	glyph.delete();
}
/**
 * called when receiver has been detached, and before redraw
 * this is for benefit of subclass notification
 */
protected void detached() {
}
/**
 * Some elements do not need to be drawn if they have no
 * visible attached children.  Returning true forces
 * the display of 'empty' nodes.
 * Subclasses should override to suppress the draw by
 * returning false; 
 */
protected boolean drawIfNoChildren() {
	return true;
}
public void expanded() {
}
/**
 * reply the body - the bit of text above the line in the box
 * reply null (or empy array) to show no body.
 * Each string in the array will apear on a separate line
 */
protected abstract String[] getBody();
public String getDescription() {return "";}
/**
 * return  a textual description of the receiver
 */
public String getExplanation() {
	return "This node is currently unexplained.";
}
protected final View getGlyphView() {
	return glyph.getGlyphView();
}
/**
 * reply the header - the bit of text above the line in the box
 * reply null to show no header
 */
protected abstract String getHeader();
/**
 * reply the menu 
 * reply null for no menu
 */
public PopupMenu getMenu() {
    PopupMenu popup = new PopupMenu();
    Menu glyphMenu = new Menu("Glyph");
    MenuItem menuItem = null;
    
    if (visible) {
        menuItem = new MenuItem(HIDE_GLYPH);
        menuItem.addActionListener(this);
        glyphMenu.add(menuItem);
    } else {
        menuItem = new MenuItem(SHOW_GLYPH);
        menuItem.addActionListener(this);
        glyphMenu.add(menuItem);
    }
    
    if (glyph.hasChildren()) {
        menuItem = new MenuItem(SHOW_CHILDREN);
        menuItem.addActionListener(this);
        glyphMenu.add(menuItem);
    }

    if (glyph.getShowBoundary()) {
        menuItem = new MenuItem(HIDE_BOUNDARY);
        menuItem.addActionListener(this);
        glyphMenu.add(menuItem);
    } else {
        menuItem = new MenuItem(SHOW_BOUNDARY);
        menuItem.addActionListener(this);
        glyphMenu.add(menuItem);
    }
    
    popup.add(glyphMenu);
    
    menuItem = new MenuItem(MI_HELP);
	menuItem.addActionListener(this);
	popup.add(menuItem);
    return popup;
}
public String getName() {
	return "";
}
public final Node getParentNode() {
	Node n = null;
	AreaGlyph g = glyph.getParent();
	if (null != g) {
		n = g.getNode();
	}
	return n;
	
}
public String getShortDescription() {return "";}
protected final View getView() {
	return getGlyphView();
}
/**
 * reply the receivers x Coordinate 
 * (will be -1 if receiver is attached)
 */
public final int getXcoord() {
	return glyph.getXcoord();
}
/**
 * reply the receivers yCoordinate 
 * (will be -1 if receiver is attached)
 */
public final int getYcoord() {
	return glyph.getYcoord();
}
/**
 * return the node highest in the hierarchy to which the 
 * receiver is attached
 */
public final Node highestAttachedNode() {
	Node n = null;
	AreaGlyph g = glyph.highestAttachedGlyph();
	if (null != g) {
		n = g.getNode();
	}
	return n;
}
/**
 * reply true if the receiver is attached to its parent
 */
public final boolean isAttached() {
	return !glyph.isDetached();
}
public boolean isCategory() {
	return false;
}
public final boolean isExpanded() {
	return glyph.isExpanded();
}
final boolean isVisible() {
	return visible;
}
public void setChildrenVisible() {
	glyph.forceChildrenVisible();
	declareChanged();
}
/**
 * set the receivers coordinates
 */
public final void setCoordinates(int x, int y) {
	glyph.setCoords(x, y);
}
public void setExpanded(boolean set) {
	glyph.setExpanded(set);
}
public final void setVisible(boolean set) {
	visible = set;
}
public String toString() {
	return getShortDescription();
}
}
